part of '../bolter_provider.dart';

mixin _PresenterProviderSingleChildWidget on SingleChildStatefulWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatefulWidget
    with _PresenterProviderSingleChildWidget {
  final P presenter;
  final bool lazy;
  final Widget child;

  const PresenterProvider(
      {Key? key,
      required this.child,
      required this.presenter,
      this.lazy = false})
      : super(key: key);

  static P of<P extends Presenter>(BuildContext context) =>
      Provider.of<P>(context, listen: false);

  @override
  PresenterProviderState<P> createState() => PresenterProviderState<P>();
}

class PresenterProviderState<P extends Presenter>
    extends SingleChildState<PresenterProvider<P>> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.endOfFrame.then(
      (_) {
        if (mounted) {
          widget.presenter.onLayout();
        }
      },
    );
    widget.presenter._bolter = BolterProvider._bolter;
    widget.presenter.initState();
  }

  @override
  Widget buildWithChild(BuildContext context, Widget? child) {
    return Provider<P>(
      lazy: widget.lazy,
      create: (ctx) {
        widget.presenter.context = context;
        widget.presenter._flutterState =
            BolterProvider.of(context)._flutterState;
        return widget.presenter;
      },
      dispose: (_, presenter) => presenter.dispose(),
      child: widget.child,
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({
    Key? key,
    required this.providers,
    required this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}
